@@ -2,14 +2,13 @@ from warnings import warn
 from s3fs import S3FileSystem
 from time import sleep
 #  from botocore.exceptions import NoCredentialsError
-from pandas import read_csv
+import pandas as pd
 from pandas import DataFrame as PandasDF
 from pickle import dump as pickle_dump
 from pickle import load as pickle_load
 from psycopg2 import connect as psycopg2_connect
 from csv import QUOTE_NONNUMERIC
-#  from pyspark.sql.dataframe import DataFrame as SparkDF
-#  from pyspark.sql import SparkSession
+
 from .S3File import S3Files
 
 
@@ -41,14 +40,53 @@ class S3:
 	def file_system(self):
 		return self._file_system
 
+	@staticmethod
+	def _get_path(path):
+		"""
+		:type path: S3Path or str
+		:rtype: str
+		"""
+		if isinstance(path, S3Path):
+			return path.path
+		else:
+			return path
+
 	def _get_absolute_path(self, path):
 		if not path.startswith(self._root):
 			path = self._root + path
 		return path
 
-	def ls(self, path, exclude_empty=False, detail=False, sort_by='path', sort_reverse=True, **kwargs):
+	def dir(self, path, exclude_empty=False, sort_by='path', **kwargs):
+		"""
+		:type path: str or Path
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
+		return self.ls(path=path, exclude_empty=exclude_empty, sort_by=sort_by, **kwargs)
+
+	def list(self, path, exclude_empty=False, sort_by='path', **kwargs):
+		"""
+		:type path: str or Path
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
+		return self.ls(path=path, exclude_empty=exclude_empty, sort_by=sort_by, **kwargs)
+
+	def ls(self, path, exclude_empty=False, sort_by='path', **kwargs):
+		"""
+		:type path: str or Path
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
-		files = self.file_system.ls(path=path, detail=detail, **kwargs)
+		files = self.file_system.ls(path=path, detail=False, **kwargs)
 
 		# if the path is only a file ls will return itself.
 		if len(files) == 1:
@@ -57,21 +95,27 @@ class S3:
 
 		if exclude_empty:
 			files = [file for file in files if self.get_size(file) > 0]
-		if detail:
-			files = S3Files(files)
-			files.sort(by=sort_by, reverse=sort_reverse)
+		#  if detail:
+		#	files = S3Files(files)
+		#	files.sort(by=sort_by, reverse=sort_reverse)
 		else:
 			if sort_by is not None:
 				files.sort()
-		return files
+
+		return [S3Path(s3=self, path=x) for x in files]
 
 	def mv(self, path1, path2, recursive=True, max_depth=None, **kwargs):
+		path1 = self._get_path(path=path1)
+		path2 = self._get_path(path=path2)
 		return self.file_system.mv(path1=path1, path2=path2, recursive=recursive, maxdepth=max_depth, **kwargs)
 
 	def cp(self, path1, path2, recursive=True, on_error=None, **kwargs):
+		path1 = self._get_path(path=path1)
+		path2 = self._get_path(path=path2)
 		return self.file_system.copy(path1=path1, path2=path2, recursive=recursive, on_error=on_error, **kwargs)
 
 	def rm(self, path, recursive=True, **kwargs):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		result = self.file_system.delete(path=path, recursive=recursive, **kwargs)
 		if self.exists(path):
@@ -79,10 +123,12 @@ class S3:
 		return result
 
 	def mkdir(self, path, **kwargs):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		return self.file_system.mkdir(path=path, **kwargs)
 
 	def tree(self, path, depth_limit=None, indentation='\t'):
+		path = self._get_path(path=path)
 		def _get_tree(_path, _depth):
 			subs = self.ls(_path)
 			if _depth == 0:
@@ -106,6 +152,7 @@ class S3:
 		print(_get_tree(_path=path, _depth=0))
 
 	def exists(self, path):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		exception = RuntimeError('No NoCredentialsError!')
 		for try_number in range(1, 5):
@@ -121,11 +168,13 @@ class S3:
 		raise exception
 
 	def write_bytes(self, path, bytes):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		with self.file_system.open(path=path, mode='wb') as f:
 			f.write(bytes)
 
 	def get_size(self, path):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		return self.file_system.size(path=path)
 
@@ -138,7 +187,7 @@ class S3:
 		:type encoding: str
 		:rtype:
 		"""
-
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		if isinstance(data, PandasDF):
 			_bytes = data.to_csv(path_or_buf=None, quoting=QUOTE_NONNUMERIC, index=index, **kwargs).encode(encoding)
@@ -149,14 +198,16 @@ class S3:
 				return data.write.csv(path=path, encoding=encoding, **kwargs)
 
 	def read_bytes(self, path):
+		path = self._get_path(path=path)
 		with self.file_system.open(path=self._root+path, mode='rb') as f:
 			result = f.read()
 		return result
 
-	def read_csv(self, path, encoding='utf-8', **kwargs):
+	def read_csv(self, path, header=True, encoding='utf-8', **kwargs):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		with self.file_system.open(self._root+path, 'rb', ) as f:
-			df = read_csv(f, encoding=encoding, **kwargs)
+			df = pd.read_csv(f, header=header, encoding=encoding, **kwargs)
 		return df
 
 	def write_pickle(self, obj, path, mode='overwite'):
@@ -167,6 +218,7 @@ class S3:
 		:type mode: str
 		:rtype: bool
 		"""
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		if self.exists(path=path):
 			if mode == 'overwrite':
@@ -188,12 +240,14 @@ class S3:
 					raise e
 
 	def read_pickle(self, path):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		with self.file_system.open(path=path, mode='rb') as f:
 			obj = pickle_load(file=f)
 		return obj
 
 	def copy_to_redshift(self, path, redshift, schema, table, truncate=False, create_table=False):
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 		if create_table:
 			data = self.read_csv(path=path)
@@ -223,6 +277,10 @@ class S3:
 
 		connection.close()
 
+	def save_parquet(self, data, path, mode='overwrite'):
+		path = self._get_path(path=path)
+		return self.write_parquet(data=data, path=path, mode=mode)
+
 	def write_parquet(self, data, path, mode='overwrite'):
 		"""
 		saves a Spark DataFrame to a path on S3 and returns the list of parquet files
@@ -231,6 +289,7 @@ class S3:
 		:type mode: str
 		:rtype: list[str]
 		"""
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 
 		if mode == 'overwrite' and self.exists(path=path):
@@ -255,13 +314,14 @@ class S3:
 		"""
 		self._spark = spark
 
-	@staticmethod
-	def get_file_name_and_extension(path):
+	@classmethod
+	def get_file_name_and_extension(cls, path):
 		"""
 		returns a file name from a path
 		:type path: str
 		:rtype: str
 		"""
+		path = cls._get_path(path=path)
 		return path.strip('/').split('/')[-1]
 
 	@classmethod
@@ -271,6 +331,7 @@ class S3:
 		:type path: str
 		:rtype: str
 		"""
+		path = cls._get_path(path=path)
 		return cls.get_file_name_and_extension(path=path).split('.')[0]
 
 	@classmethod
@@ -287,12 +348,20 @@ class S3:
 			return None
 
 	def is_file(self, path):
+		path = self._get_path(path=path)
 		return self.file_system.isfile(path=path)
 
 	def is_dir(self, path):
+		path = self._get_path(path=path)
 		return self.file_system.isdir(path)
 
+	def is_directory(self, path):
+		path = self._get_path(path=path)
+		return self.is_dir(path=path)
+
 	def rename(self, path1, path2):
+		path1 = self._get_path(path=path1)
+		path2 = self._get_path(path=path2)
 		self.file_system.rename(path1=path1, path2=path2)
 
 	@property
@@ -305,12 +374,17 @@ class S3:
 		:type path: str
 		:rtype: bool
 		"""
+		path = self._get_path(path=path)
 		if self.get_size(path=path) == 0:
 			return False
 		else:
 			n_and_e = self.get_file_name_and_extension(path=path).lower()
 			return n_and_e.startswith('part-') and n_and_e.endswith('.parquet')
 
+	def load_parquet(self, path, spark=None, parallel=True):
+		path = self._get_path(path=path)
+		return self.read_parquet(path=path, spark=spark, parallel=parallel)
+
 	def read_parquet(self, path, spark=None, parallel=True):
 		"""
 		reads parquet files inside a path and returns the data
@@ -319,6 +393,7 @@ class S3:
 		:type parallel: bool
 		:rtype: SparkDF
 		"""
+		path = self._get_path(path=path)
 		if spark is None:
 			spark = self.spark
 
@@ -326,6 +401,7 @@ class S3:
 		if len(files) == 1:
 			file = files[0]
 			if self.get_size(file) > 0:
+				file = self._get_path(file)
 				if not self.is_parquet_file(path=file):
 					print(f'"{file}" does not appear to be a parquet file!')
 				return spark.read.parquet(file)
@@ -355,6 +431,7 @@ class S3:
 		:type mode: str
 		:rtype: bool
 		"""
+		path = self._get_path(path=path)
 		if path.endswith('.pickle'):
 			self.write_pickle(obj=obj, path=path, mode=mode)
 
@@ -375,6 +452,7 @@ class S3:
 		:type spark: pyspark.sql.session.SparkSession
 		:rtype: SparkDF or PandasDF or obj
 		"""
+		path = self._get_path(path=path)
 		path = self._get_absolute_path(path)
 
 		if path.endswith('.parquet'):
@@ -399,13 +477,9 @@ class S3:
 	def __add__(self, other):
 		return self.__truediv__(other)
 
-	list = ls
 	move = mv
-	dir = ls
 	copy = cp
 	delete = rm
-	load_parquet = read_parquet
-	save_parquet = write_parquet
 	load_pickle = read_pickle
 	save_pickle = write_pickle
 
@@ -417,16 +491,34 @@ class S3Path:
 		:type path: str
 		"""
 		if path.startswith(s3.root):
-			warn(f'path "{path}" includes S3 root "{s3.root}"!')
+			#  warn(f'path "{path}" includes S3 root "{s3.root}"!')
 			path = path[len(s3.root):]
 
 		self._s3 = s3
 		self._path = path
+		self._name_and_extension = self._path.rstrip('/').split('/')[-1]
+		if '.' in self.name_and_extension:
+			self._name, self._extension = self.name_and_extension.rsplit('.')
+		else:
+			self._name = self.name_and_extension
+			self._extension = None
 
 	@property
 	def path(self):
 		return self._path
 
+	@property
+	def name_and_extension(self):
+		return self._name_and_extension
+
+	@property
+	def name(self):
+		return self._name
+
+	@property
+	def extension(self):
+		return self._extension
+
 	def __truediv__(self, other):
 		"""
 		:type other: str
@@ -451,8 +543,12 @@ class S3Path:
 			return self.__truediv__(other)
 
 	def __repr__(self):
+		return self.full_path
+
+	@property
+	def full_path(self):
 		path = self._path
-		if path.lower().startswith('s3'):
+		if path.lower().startswith('s3://'):
 			return path
 		else:
 			return f's3://{path}'
@@ -478,17 +574,53 @@ class S3Path:
 		return self.s3.save(obj=obj, path=self._path, mode=mode)
 
 	def ls(self, **kwargs):
+		"""
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
 		return self.s3.ls(path=self._path, **kwargs)
 
+	def dir(self, **kwargs):
+		"""
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
+		return self.ls(**kwargs)
+
+	def list(self, **kwargs):
+		"""
+		:type exclude_empty: bool
+		:type sort_by: str
+		:type kwargs: dict
+		:rtype: list[Path]
+		"""
+		return self.ls(**kwargs)
+
 	def is_file(self):
 		return self.s3.is_file(path=self._path)
 
 	def is_dir(self):
 		return self.s3.is_dir(path=self._path)
 
+	def is_directory(self):
+		return self.is_dir()
+
 	def make_directory(self):
 		return self.s3.mkdir(path=self._path)
 
+	def mkdir(self):
+		return self.make_directory()
+
+	def make_dir(self):
+		return self.make_directory()
+
+	def md(self):
+		return self.make_directory()
+
 	@property
 	def size(self):
 		return self.s3.get_size(path=self._path)
@@ -499,6 +631,9 @@ class S3Path:
 	def delete(self):
 		return self.s3.delete(path=self._path)
 
+	def rm(self):
+		return self.delete()
+
 	def get_num_files(self):
 		if self.is_dir():
 			return len(self.s3.file_system.ls(path=self.path))
@@ -507,11 +642,3 @@ class S3Path:
 
 	def is_empty(self):
 		return self.get_num_files() == 0
-
-	dir = ls
-	list = ls
-	mkdir = make_directory
-	make_dir = make_directory
-	md = make_directory
-	is_directory = is_dir
-	rm = delete
