@@ -1,2 +1,2 @@
-from .S3 import S3
-from .redshift.Redshift import Redshift
\ No newline at end of file
+from .S3 import S3, S3Path
+from .redshift.Redshift import Redshift
